# -*- coding: utf8 -*-
from __future__ import unicode_literals


import csv
import datetime
import logging

import six
from six import BytesIO, StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, String

logger = logging.getLogger("driverng.lc480")


class LC480Base(Splitted):

    SEPARATOR = ","
    ENCODING = "utf-8"


class LC480Result(object):

    def parse(self, line, fields):
        for index, field in enumerate(fields):
            setattr(self, field.name, line[index])


class LC480ResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: LC480Result
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode('utf-8'))
        else:
            buf = StringIO(data)
        self.fields = []
        reader = csv.reader(buf, delimiter=str(';'))
        self.date_resu = None
        self.instrument_serial_number = None
        idx = 0
        for row in reader:
            if idx == 0:
                for col in row:
                    self.fields.append(String(col))
            else:
                if six.PY2:
                    line = [c.decode('utf-8') for c in row]  # PY2 csv reader est imcompatible avec unicode
                else:
                    line = row
                if not len(line):
                    continue
                if len(line) == len(self.fields):
                    obj = self.add(type_message=MessageV2.RESULT)
                    obj.parse(line, self.fields)
            idx += 1


class Driver(MessageV2):

    SPEC = {
        "code": "lc480",
        "label": "LC480  "
    }

    SAMPLE = "SampleName"
    POSITION = "Position"
    ANALYSE = "Analysis Name"
    RESUS = ["CrossingPoint", "Call"]

    def parse(self, data):
        obj = LC480ResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            sample_id = getattr(m, self.SAMPLE, None)
            if not sample_id:
                continue
            controle = sample_id.startswith("CQ")
            pos = getattr(m, self.POSITION, "")
            analyse = getattr(m, self.ANALYSE, "")

            sample = worklist.get_or_create(
                sample_id=sample_id,
                rack=pos,
                type="control" if controle else "patient"
            )
            if controle:
                sample.control_lot = sample_id
            for attr in self.RESUS:
                resu = getattr(m, attr).replace(',', '.').strip()
                if resu:
                    resultat = sample.results.add(
                        code="{}_{}".format(analyse, attr),
                        value=getattr(m, attr).replace(',', '.'),
                    )
                if controle:
                    resultat.date = datetime.datetime.now().replace(second=0, microsecond=0)

        return worklist
